/*
 * File operations supporting an Ext2-compatible filesystem.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <mutex>

#include <ext2fs/ext2fs.h>

#include <systypes/base.h>
#include <systypes/user.h>



/* Directory iteration helper type. */

class Ext2FileOperations;

struct Ext2DirectoryIteration
{
    Ext2FileOperations *ops;
    int (*func)(struct ext2_dir_entry *, int, int, char *, void *);
    void *priv_data;
};



/* An Ext2 file operations collection. */

class Ext2FileOperations
{
protected:
    std::mutex _lock;
    ext2_filsys _fs;

public:
    explicit Ext2FileOperations(ext2_filsys fs)
    : _fs(fs)
    {
    }

    bool is_directory(ext2_ino_t ino_file);

    bool is_file(ext2_ino_t ino_file);

    void close_file(ext2_file_t file);

    long create_file(ext2_ino_t ino_parent, const char *filename, user_t user,
                     ext2_ino_t *ino);

    long find_file(const char *path, ext2_ino_t *ino, const char **remaining);

    long open_file(ext2_ino_t ino, ext2_file_t *file);

    long remove(ext2_ino_t ino);

    long rename(ext2_ino_t source,
                ext2_ino_t source_parent, const char *source_basename,
                ext2_ino_t target_parent, const char *target_basename);

    long unlink(ext2_ino_t ino_parent, ext2_ino_t ino);

    /* File property methods. */

    offset_t get_size(ext2_file_t file);

    void set_size(ext2_file_t file, offset_t size);

    /* File access methods. */

    offset_t read_file(ext2_file_t file, offset_t filepos, void *addr, offset_t size);

    void write_file(ext2_file_t file, offset_t filepos, const void *addr, offset_t size);

    /* Directory access. */

    long directory_iterate(ext2_ino_t dir,
                           int func(struct ext2_dir_entry *, int, int, char *, void *),
                           void *priv_data);

    long read_inode(ext2_ino_t ino_file, struct ext2_inode *inode);

    /* Callback method. */

    int directory_iterate_fn(struct ext2_dir_entry *dir_entry,
                             int offset, int blocksize,
                             char *buf, struct Ext2DirectoryIteration *priv_data);
};

// vim: tabstop=4 expandtab shiftwidth=4
